<?php

namespace Drupal\custom_field\Plugin\CustomFieldType;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Component\Utility\NestedArray;

/**
 * Plugin implementation of the 'Map (Key Value)' customfield type.
 *
 * Simple textfield customfield widget. Value renders as it is entered by the
 * user.
 *
 * @CustomFieldType(
 *   id = "map_key_value",
 *   label = @Translation("Map (Key Value)"),
 *   description = @Translation(""),
 *   category = @Translation("General"),
 *   data_types = {
 *     "map",
 *   },
 * )
 */
class MapKeyValue extends MapBase {

  /**
   * {@inheritdoc}
   */
  public function widget(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {

    // Get the base form element properties.
    $element = parent::widget($items, $delta, $element, $form, $form_state);
    $settings = $this->widget_settings['settings'];
    $element['#default_value'] = empty($element['#default_value']) ? ['' => ''] : $element['#default_value'];
    $element['#element_validate'] = [[static::class, 'validateArrayValues']];
    $options_wrapper_id = 'options-wrapper-' . $delta . $this->name;
    $element['#prefix'] = '<div id="' . $options_wrapper_id . '">';
    $element['#suffix'] = '</div>';
    $values = $form_state->getValues();
    $fieldName = $items->getFieldDefinition()->getName();
    if (empty($values)) {
      $mapItems = $element['#default_value'];
    }
    else {
      $mapItems = $values[$fieldName][$delta][$this->name];
    }
    // Convert to multi-dimensional array for manipulation.
    $mapList = [];
    foreach ($mapItems as $key => $value) {
      $mapList[] = ['key' => $key, 'value' => $value];
    }
    if ($form_state->isRebuilding()) {
      $trigger = $form_state->getTriggeringElement();
      if ($trigger['#name'] == 'add_item:' . $options_wrapper_id) {
        $mapList[] = ['key'=> '', 'value' => ''];
        $form_state->set('add', NULL);
      }
      if ($form_state->get('remove')) {
        $remove = $form_state->get('remove');
        if ($remove['name'] === 'remove:' . $options_wrapper_id . $trigger['#delta']) {
          unset($mapList[$remove['key']]);
          $form_state->set('remove', NULL);
        }
      }
    }
    $element['data'] = [
      '#type' => 'table',
      '#header' => [
        t('Key'),
        t('Value'),
        '',
      ],
      '#tree' => TRUE,
    ];

    foreach ($mapList as $key => $value) {
      $element['data'][$key]['key'] = [
        '#type' => 'textfield',
        '#title' => t('Key'),
        '#title_display' => 'invisible',
        '#default_value' => $value['key'],
        '#required' => TRUE,
      ];
      $element['data'][$key]['value'] = [
        '#type' => 'textfield',
        '#title' => t('Value'),
        '#title_display' => 'invisible',
        '#default_value' => $value['value'],
        //'#required' => TRUE,
      ];
      $element['data'][$key]['remove'] = [
        '#type' => 'submit',
        '#value' => t('Remove'),
        '#submit' => [get_class($this) . '::removeSubmit'],
        '#name' => 'remove:' . $options_wrapper_id . $key,
        '#delta' => $key,
        '#ajax' => [
          'callback' => [$this, 'actionCallback'],
          'wrapper' => $options_wrapper_id,
        ],
      ];
    }
    $element['add_item'] = [
      '#type' => 'submit',
      '#value' => t('Add item'),
      '#submit' => [get_class($this) . '::addItem'],
      '#name' => 'add_item:' . $options_wrapper_id,
      '#ajax' => [
        'callback' => [$this, 'actionCallback'],
        'wrapper' => $options_wrapper_id,
      ],
    ];

    return $element;

  }

  /**
   * The #element_validate callback for map field array values.
   *
   * @param $element
   *   An associative array containing the properties and children of the
   *   generic form element.
   * @param $form_state
   *   The current state of the form for the form this element belongs to.
   *
   * @see \Drupal\Core\Render\Element\FormElement::processPattern()
   */
  public static function validateArrayValues($element, FormStateInterface $form_state) {
    $values = $element['#value'];
    if (isset($values['data']) && is_array($values['data'])) {
      $unique_keys = [];
      $saved_values = [];
      foreach ($values['data'] as $value) {
        // Make sure each key is unique.
        if (in_array($value['key'], $unique_keys)) {
          $form_state->setError($element, t('All keys must be unique.'));
          break;
        }
        else {
          $unique_keys[] = $value['key'];
          $saved_values[$value['key']] = $value['value'];
        }
      }
      $form_state->setValueForElement($element, $saved_values);
    }
  }

  /**
   * Submit handler for the "add item" button.
   *
   */
  public static function addItem(array &$form, FormStateInterface $form_state) {
    $form_state->set('add', $form_state->getTriggeringElement()['#name']);
    $form_state->setRebuild();
  }

  /**
   * Submit handler for the "remove item" button.
   *
   */
  public static function removeSubmit(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $form_state->set('remove', ['name' => $trigger['#name'], 'key' => $trigger['#delta']]);
    $form_state->setRebuild();
  }

  /**
   * Callback for both ajax-enabled buttons.
   *
   * Selects and returns the fieldset with the names in it.
   */
  public function actionCallback(array &$form, FormStateInterface $form_state) {
    $parents = $form_state->getTriggeringElement()['#array_parents'];
    return $form[$parents[0]][$parents[1]][$parents[2]][$parents[3]];
  }

}
